#!/usr/bin/php

<?php

// Copyright (C) 2007 Cygnus <cygnus2 at post.cz>
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or (at
// your option) any later version.
// 
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA

define ("CFAIL","\033[1;31;40m !\033[0m ");
define ("COK","\033[1;32;40m *\033[0m ");
define ("CWARN","\033[1;33;40m ?\033[0m ");
define ("CTRY","\033[1;34;40m -\033[0m ");

define ("CRED","\033[1;31;40m");
define ("CGRN","\033[1;32;40m");
define ("CYELL","\033[1;33;40m");
define ("CBLUE","\033[1;34;40m");
define ("CMAGN","\033[1;35;40m");
define ("CCYAN","\033[1;36;40m");
define ("CWHITE","\033[1;37;40m");

define ("CDEF","\033[0m");

// configuration
$cfg_infile1		= './driver_desktop_alfigraf_before_initialization.bin';
$cfg_infile2		= './driver_desktop_alfigraf_after_initialization.bin';
$cfg_first_address	= 46520;		// first address in Z80 memory

$cfg_number_per_line	= 16;

// --------------------------------------------------------------------------------------

function to_hex ($b)
{
	// for bytes
	global $cfg_hex_caps;

	if ($cfg_hex_caps)
		$hextab = array ('0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F');
	else
		$hextab = array ('0','1','2','3','4','5','6','7','8','9','a','b','c','d','e','f');

	$h = floor($b / 16);
	$l = $b - ($h * 16);

	return ($hextab[$h].$hextab[$l]);
}

function to_hex_16b ($word)
{
	// for addresses
	$h = floor ($word / 256);
	$l = $word - 256 * $h;

	return (to_hex($h).to_hex($l));
}

// --------------------------------------------------------------------------------------
function compare_binary_files ($in1, $in2)
{
	global $cfg_infile1;
	global $cfg_infile2;
	global $cfg_first_address;
	global $cfg_number_per_line;

	$diffs = 0;
	$error = 0;
	$address = $cfg_first_address;
	$line_counter = 0;
	$ln_left = '';
	$ln_right = '';
	
	$message = '';		// for final result message

	while (1)
	{
		$ch1 = fread($in1, 1);
		$ch2 = fread($in2, 1);

		// Z80 address space limit
		if ($address > 65535)
		{
			echo CFAIL."Z80 16 bit address space exceeded!\n";
			$error = 1;
			break;
		}

		// end of both files at same byte?
		if (($ch1 === false || feof($in1)) && ($ch2 === false || feof($in2)))
		{
			if ($diffs == 0)
			{
				$message = COK."Both files are identical.\n".CDEF;
			}
			else
			{
				$message = CRED."Number of differences ".CYELL.$diffs."\n";
			}

			// finish printing if last line is incomplete
			if ($line_counter > 0)
			{
				for ($n = $line_counter; $n < $cfg_number_per_line; $n++)
				{
					$ln_left .= CDEF.'   ';
					$ln_right .= CDEF.'   ';
				}
				echo $ln_left.' -  '.$ln_right."\n";
			}

			echo $message;
			break;
		}
		else
		{
			if ($ch1 === false || feof($in1))
			{
 				$message = CWARN."File ".$cfg_infile2." is longer.\n".CDEF;
				$error = 1;
			}

			if ($ch2 === false || feof($in2))
			{
				$message = CWARN."File ".$cfg_infile1." is longer.\n".CDEF;
				$error = 1;
			}

			if ($error == 1)
			{
				// finish printing if last line is incomplete
				if ($line_counter > 0)
				{
					for ($n = $line_counter; $n < $cfg_number_per_line; $n++)
					{
						$ln_left .= CDEF.'   ';
						$ln_right .= CDEF.'   ';
					}
					echo $ln_left.' -  '.$ln_right."\n";
				}

				if ($diffs == 0)
				{
					$message = COK."Both files are identical.\n".CDEF;
				}
				else
				{
					$message = CRED."Number of differences ".CYELL.$diffs."\n";
				}
				
				echo $message;
				break;
			}
		}

		// print address only once for one line
		if ($line_counter == 0)
		{
			if ($address < 9) echo ' ';
			if ($address < 99) echo ' ';
			if ($address < 999) echo ' ';
			if ($address < 9999) echo ' ';
			echo CWHITE.$address.'  #'.to_hex_16b($address).CDEF.'  ';
		}

		// print line, compare bytes, print bytes, status ...
		$b1 = unpack ('C', $ch1);	// unpack as unsigned char = byte
		$b2 = unpack ('C', $ch2);	// unpack as unsigned char = byte

		if ($b1[1] == $b2[1])
			$color = CDEF;
		else
		{
			$color = CRED;
			$diffs++;
		}

		$ln_left .= ($color.to_hex ($b1[1]).CDEF.' ');
		$ln_right .= ($color.to_hex ($b2[1]).CDEF.' ');

		$address++;
		$line_counter++;

		if (($line_counter + 1) > $cfg_number_per_line)
		{
			$line_counter = 0;
			echo $ln_left.' -  '.$ln_right."\n";
			$ln_left = '';
			$ln_right = '';
		}
	}
}

// --------------------------------------------------------------------------------------

$in1 = fopen ($cfg_infile1, "r");
$in2 = fopen ($cfg_infile2, "r");

if ($in1 && $in2)
{
	echo 'File 1 = '.CWHITE.$cfg_infile1.CDEF."\n";
	echo 'File 2 = '.CWHITE.$cfg_infile2.CDEF."\n";
	
	if ($cfg_number_per_line < 1) $cfg_number_per_line = 1;

	compare_binary_files ($in1, $in2);
}
else
{
	if (!$in1) echo CFAIL."Warning: opening first file \"$cfg_infile1\" failed.\n".CDEF;
	if (!$in2) echo CFAIL."Warning: opening second file \"$cfg_infile2\" failed.\n".CDEF;
}

if ($in1) fclose ($in1);
if ($in2) fclose ($in2);

return (0);

?>
